#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""ONF_website.shared.view.skin.map module

This module defines a custom renderer which can be used to render view items as
map markers.
"""

import json

from zope.interface import Interface
from zope.schema.fieldproperty import FieldProperty

from onf_website.component.location import ILocationInfo
from onf_website.shared.view.skin.interfaces import IONFViewItemsPortletMapRendererSettings
from pyams_content.shared.view.portlet import IViewItemsPortletSettings
from pyams_default_theme.interfaces import IContentNavigationIllustration, IContentTitle
from pyams_default_theme.shared.view.portlet import BaseViewItemsPortletRenderer
from pyams_file.interfaces import IThumbnails
from pyams_gis.configuration import MapConfiguration
from pyams_gis.interfaces.configuration import IMapConfiguration
from pyams_gis.interfaces.utility import IMapManager
from pyams_i18n.interfaces import II18n
from pyams_portal.interfaces import IPortalContext, IPortletRenderer
from pyams_skin.layer import IPyAMSLayer, IPyAMSUserLayer
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config
from pyams_utils.factory import factory_config
from pyams_utils.interfaces import ICacheKeyValue
from pyams_utils.registry import get_utility


__docformat__ = 'restructuredtext'

from onf_website import _
from pyams_utils.url import absolute_url


@factory_config(provided=IONFViewItemsPortletMapRendererSettings)
class ONFViewItemsPortletMapRendererSettings(MapConfiguration):
    """View items portlet renderer settings"""

    _use_default_map_configuration = FieldProperty(
        IONFViewItemsPortletMapRendererSettings['use_default_map_configuration'])

    map_height = FieldProperty(IONFViewItemsPortletMapRendererSettings['map_height'])
    display_illustrations = FieldProperty(
        IONFViewItemsPortletMapRendererSettings['display_illustrations'])

    @property
    def use_default_map_configuration(self):
        return self._use_default_map_configuration

    @use_default_map_configuration.setter
    def use_default_map_configuration(self, value):
        self._use_default_map_configuration = value

    @property
    def no_use_default_map_configuration(self):
        return not bool(self.use_default_map_configuration)

    @no_use_default_map_configuration.setter
    def no_use_default_map_configuration(self, value):
        self.use_default_map_configuration = not bool(value)

    @property
    def configuration(self):
        if self.use_default_map_configuration:
            manager = get_utility(IMapManager)
            return IMapConfiguration(manager).get_configuration()
        return self.get_configuration()


@adapter_config(name='ONF::view.map',
                context=(IPortalContext, IPyAMSLayer, Interface, IViewItemsPortletSettings),
                provides=IPortletRenderer)
@template_config(template='templates/map.pt', layer=IPyAMSUserLayer)
class ONFViewItemsPortletMapRenderer(BaseViewItemsPortletRenderer):
    """View items portlet map renderer"""

    label = _("ONF: map view with markers")
    weight = 20

    settings_interface = IONFViewItemsPortletMapRendererSettings

    def get_map_configuration(self):
        return json.dumps(self.renderer_settings.configuration)

    def get_markers(self, results):
        config = {
            'icon': {
                'url': '/--static--/onf_website/images/leaflet-marker.png',
                'size': [47, 59],
                'anchor': [23, 58]
            },
            'clusterClass': 'onfCluster',
            'tooltipClass': 'onfTooltip',
            'markers': []
        }
        append = config['markers'].append
        registry = self.request.registry
        display_illustrations = self.renderer_settings.display_illustrations
        for result in results:
            location_info = ILocationInfo(result, None)
            if (location_info is None) or not location_info.locations:
                continue
            for location in location_info.locations:
                position = location.to_json()
                if position is None:
                    continue
                href = self.get_url(result)
                if href is None:
                    continue
                marker = {
                    'id': ICacheKeyValue(result),
                    'label': registry.queryMultiAdapter((result, self.request),
                                                        IContentTitle),
                    'href': href,
                    'point': position
                }
                if display_illustrations:
                    illustration = registry.queryMultiAdapter((result, self.request),
                                                              IContentNavigationIllustration)
                    if illustration:
                        img = II18n(illustration).query_attribute('data', request=self.request)
                        if img:
                            thumbnail = IThumbnails(img).get_thumbnail('256x256')
                            width, height = thumbnail.get_image_size()
                            marker['img'] = {
                                'src': absolute_url(thumbnail, request=self.request),
                                'w': width,
                                'h': height
                            }
                append(marker)
        return json.dumps(config)
